@@ -48,39 +48,8 @@
 #include <X11/extensions/Xfixes.h>
 #include <X11/extensions/Xrender.h>
 
-#if COMPOSITE_MAJOR > 0 || COMPOSITE_MINOR >= 2
-#define HAVE_NAME_WINDOW_PIXMAP 1
-#endif
-
-#if COMPOSITE_MAJOR > 0 || COMPOSITE_MINOR >= 3
-#define HAVE_COW 1
-#else
-/* Don't have a cow man...HAAHAAHAA */
-#endif
-
 #define USE_IDLE_REPAINT 1
 
-#ifdef HAVE_COMPOSITE_EXTENSIONS
-static inline gboolean
-composite_at_least_version (MetaDisplay *display,
-                            int maj, int min)
-{
-  static int major = -1;
-  static int minor = -1;
-
-  if (major == -1)
-    meta_display_get_compositor_version (display, &major, &minor);
-
-  return (major > maj || (major == maj && minor >= min));
-}
-
-#define have_name_window_pixmap(display) \
-  composite_at_least_version (display, 0, 2)
-#define have_cow(display) \
-  composite_at_least_version (display, 0, 3)
-
-#endif
-
 typedef enum _MetaCompWindowType
 {
   META_COMP_WINDOW_NORMAL,
@@ -178,7 +147,6 @@ typedef struct _MetaCompWindow
   Window id;
   XWindowAttributes attrs;
 
-#ifdef HAVE_NAME_WINDOW_PIXMAP
   Pixmap back_pixmap;
 
   /* When the window is shaded back_pixmap will be replaced with the pixmap
@@ -186,7 +154,6 @@ typedef struct _MetaCompWindow
      so that we can still see what the window looked like when it is needed
      for the _get_window_pixmap function */
   Pixmap shaded_back_pixmap;
-#endif
 
   int mode;
 
@@ -204,6 +171,7 @@ typedef struct _MetaCompWindow
   Picture shadow_pict;
 
   XserverRegion border_size;
+  XserverRegion window_size;
   XserverRegion extents;
 
   Picture shadow;
@@ -592,16 +560,96 @@ make_shadow (MetaDisplay   *display,
   return ximage;
 }
 
+double shadow_offsets_x[LAST_SHADOW_TYPE] = {SHADOW_SMALL_OFFSET_X,
+                                             SHADOW_MEDIUM_OFFSET_X,
+                                             SHADOW_LARGE_OFFSET_X};
+double shadow_offsets_y[LAST_SHADOW_TYPE] = {SHADOW_SMALL_OFFSET_Y,
+                                             SHADOW_MEDIUM_OFFSET_Y,
+                                             SHADOW_LARGE_OFFSET_Y};
+
+static XserverRegion
+cairo_region_to_xserver_region (Display        *xdisplay,
+                                cairo_region_t *region)
+{
+  int n_rects, i;
+  XRectangle *rects;
+  XserverRegion xregion;
+
+  n_rects = cairo_region_num_rectangles (region);
+  rects = g_new (XRectangle, n_rects);
+
+  for (i = 0; i < n_rects; i++)
+    {
+      cairo_rectangle_int_t rect;
+
+      cairo_region_get_rectangle (region, i, &rect);
+
+      rects[i].x = rect.x;
+      rects[i].y = rect.y;
+      rects[i].width = rect.width;
+      rects[i].height = rect.height;
+    }
+
+  xregion = XFixesCreateRegion (xdisplay, rects, n_rects);
+  g_free (rects);
+
+  return xregion;
+}
+
+static void
+shadow_picture_clip (Display          *xdisplay,
+                     Picture           shadow_picture,
+                     MetaCompWindow   *cw,
+                     MetaFrameBorders  borders,
+                     int               width,
+                     int               height)
+{
+  int shadow_dx;
+  int shadow_dy;
+  cairo_region_t *visible_region;
+  XRectangle rect;
+  XserverRegion region1;
+  XserverRegion region2;
+
+  if (!cw->window)
+    return;
+
+  visible_region = meta_window_get_frame_bounds (cw->window);
+
+  if (!visible_region)
+    return;
+
+  shadow_dx = -1 * (int) shadow_offsets_x [cw->shadow_type] - borders.invisible.left;
+  shadow_dy = -1 * (int) shadow_offsets_y [cw->shadow_type] - borders.invisible.top;
+
+  rect.x = 0;
+  rect.y = 0;
+  rect.width = width;
+  rect.height = height;
+
+  region1 = XFixesCreateRegion (xdisplay, &rect, 1);
+  region2 = cairo_region_to_xserver_region (xdisplay, visible_region);
+
+  XFixesTranslateRegion (xdisplay, region2,
+                         shadow_dx, shadow_dy);
+
+  XFixesSubtractRegion (xdisplay, region1, region1, region2);
+  XFixesSetPictureClipRegion (xdisplay, shadow_picture, 0, 0, region1);
+
+  XFixesDestroyRegion (xdisplay, region1);
+  XFixesDestroyRegion (xdisplay, region2);
+}
+
 static Picture
-shadow_picture (MetaDisplay   *display,
-                MetaScreen    *screen,
-                MetaShadowType shadow_type,
-                double         opacity,
-                Picture        alpha_pict,
-                int            width,
-                int            height,
-                int           *wp,
-                int           *hp)
+shadow_picture (MetaDisplay      *display,
+                MetaScreen       *screen,
+                MetaCompWindow   *cw,
+                double            opacity,
+                MetaFrameBorders  borders,
+                int               width,
+                int               height,
+                int              *wp,
+                int              *hp)
 {
   Display *xdisplay = meta_display_get_xdisplay (display);
   XImage *shadow_image;
@@ -610,7 +658,7 @@ shadow_picture (MetaDisplay   *display,
   Window xroot = meta_screen_get_xroot (screen);
   GC gc;
 
-  shadow_image = make_shadow (display, screen, shadow_type,
+  shadow_image = make_shadow (display, screen, cw->shadow_type,
                               opacity, width, height);
   if (!shadow_image)
     return None;
@@ -624,8 +672,7 @@ shadow_picture (MetaDisplay   *display,
     }
 
   shadow_picture = XRenderCreatePicture (xdisplay, shadow_pixmap,
-                                         XRenderFindStandardFormat (xdisplay,
-PictStandardA8),
+                                         XRenderFindStandardFormat (xdisplay, PictStandardA8),
                                          0, 0);
   if (!shadow_picture)
     {
@@ -634,6 +681,9 @@ PictStandardA8),
       return None;
     }
 
+  shadow_picture_clip (xdisplay, shadow_picture, cw, borders,
+                       shadow_image->width, shadow_image->height);
+
   gc = XCreateGC (xdisplay, shadow_pixmap, 0, 0);
   if (!gc)
     {
@@ -950,12 +1000,7 @@ window_has_shadow (MetaCompWindow *cw)
   return FALSE;
 }
 
-double shadow_offsets_x[LAST_SHADOW_TYPE] = {SHADOW_SMALL_OFFSET_X,
-                                             SHADOW_MEDIUM_OFFSET_X,
-                                             SHADOW_LARGE_OFFSET_X};
-double shadow_offsets_y[LAST_SHADOW_TYPE] = {SHADOW_SMALL_OFFSET_Y,
-                                             SHADOW_MEDIUM_OFFSET_Y,
-                                             SHADOW_LARGE_OFFSET_Y};
+
 static XserverRegion
 win_extents (MetaCompWindow *cw)
 {
@@ -971,21 +1016,34 @@ win_extents (MetaCompWindow *cw)
 
   if (cw->needs_shadow)
     {
+      MetaFrameBorders borders;
       XRectangle sr;
 
-      cw->shadow_dx = shadow_offsets_x [cw->shadow_type];
-      cw->shadow_dy = shadow_offsets_y [cw->shadow_type];
+      meta_frame_borders_clear (&borders);
+
+      if (cw->window)
+        {
+          MetaFrame *frame = meta_window_get_frame (cw->window);
+
+          if (frame)
+            meta_frame_calc_borders (frame, &borders);
+        }
+
+      cw->shadow_dx = (int) shadow_offsets_x [cw->shadow_type] + borders.invisible.left;
+      cw->shadow_dy = (int) shadow_offsets_y [cw->shadow_type] + borders.invisible.top;
 
       if (!cw->shadow)
         {
           double opacity = SHADOW_OPACITY;
+          int invisible_width = borders.invisible.left + borders.invisible.right;
+          int invisible_height = borders.invisible.top + borders.invisible.bottom;
+
           if (cw->opacity != (guint) OPAQUE)
             opacity = opacity * ((double) cw->opacity) / ((double) OPAQUE);
 
-          cw->shadow = shadow_picture (display, screen, cw->shadow_type,
-                                       opacity, cw->alpha_pict,
-                                       cw->attrs.width + cw->attrs.border_width * 2,
-                                       cw->attrs.height + cw->attrs.border_width * 2,
+          cw->shadow = shadow_picture (display, screen, cw, opacity, borders,
+                                       cw->attrs.width - invisible_width + cw->attrs.border_width * 2,
+                                       cw->attrs.height - invisible_height + cw->attrs.border_width * 2,
                                        &cw->shadow_width, &cw->shadow_height);
         }
 
@@ -1033,6 +1091,42 @@ border_size (MetaCompWindow *cw)
   XFixesTranslateRegion (xdisplay, border,
                          cw->attrs.x + cw->attrs.border_width,
                          cw->attrs.y + cw->attrs.border_width);
+
+  return border;
+}
+
+static XserverRegion
+window_size (MetaCompWindow *cw)
+{
+  MetaScreen *screen = cw->screen;
+  MetaDisplay *display = meta_screen_get_display (screen);
+  Display *xdisplay = meta_display_get_xdisplay (display);
+  cairo_region_t *visible_region;
+  XserverRegion visible = None;
+  XserverRegion border;
+
+  if (cw->window)
+    {
+      visible_region = meta_window_get_frame_bounds (cw->window);
+
+      if (visible_region)
+        visible = cairo_region_to_xserver_region (xdisplay, visible_region);
+    }
+
+  border = border_size (cw);
+
+  if (visible != None)
+    {
+      XFixesTranslateRegion (xdisplay, visible,
+                         cw->attrs.x + cw->attrs.border_width,
+                         cw->attrs.y + cw->attrs.border_width);
+
+      XFixesIntersectRegion (xdisplay, visible, visible, border);
+      XFixesDestroyRegion (xdisplay, border);
+
+      return visible;
+    }
+
   return border;
 }
 
@@ -1061,21 +1155,21 @@ get_window_picture (MetaCompWindow *cw)
   XRenderPictureAttributes pa;
   XRenderPictFormat *format;
   Drawable draw;
+  int error_code;
 
   draw = cw->id;
 
   meta_error_trap_push (display);
 
-#ifdef HAVE_NAME_WINDOW_PIXMAP
-  if (have_name_window_pixmap (display))
-    {
-      if (cw->back_pixmap == None)
-        cw->back_pixmap = XCompositeNameWindowPixmap (xdisplay, cw->id);
+  if (cw->back_pixmap == None)
+    cw->back_pixmap = XCompositeNameWindowPixmap (xdisplay, cw->id);
 
-      if (cw->back_pixmap != None)
-        draw = cw->back_pixmap;
-    }
-#endif
+  error_code = meta_error_trap_pop_with_return (display, FALSE);
+  if (error_code != 0)
+    cw->back_pixmap = None;
+
+  if (cw->back_pixmap != None)
+    draw = cw->back_pixmap;
 
   format = get_window_format (cw);
   if (format)
@@ -1084,13 +1178,13 @@ get_window_picture (MetaCompWindow *cw)
 
       pa.subwindow_mode = IncludeInferiors;
 
+      meta_error_trap_push (display);
       pict = XRenderCreatePicture (xdisplay, draw, format, CPSubwindowMode, &pa);
       meta_error_trap_pop (display, FALSE);
 
       return pict;
     }
 
-  meta_error_trap_pop (display, FALSE);
   return None;
 }
 
@@ -1211,6 +1305,12 @@ paint_windows (MetaScreen   *screen,
               cw->border_size = None;
             }
 
+          if (cw->window_size)
+            {
+              XFixesDestroyRegion (xdisplay, cw->window_size);
+              cw->window_size = None;
+            }
+
 #if 0
           if (cw->extents)
             {
@@ -1223,6 +1323,9 @@ paint_windows (MetaScreen   *screen,
       if (cw->border_size == None)
         cw->border_size = border_size (cw);
 
+      if (cw->window_size == None)
+        cw->window_size = window_size (cw);
+
       if (cw->extents == None)
         cw->extents = win_extents (cw);
 
@@ -1230,22 +1333,10 @@ paint_windows (MetaScreen   *screen,
         {
           int x, y, wid, hei;
 
-#ifdef HAVE_NAME_WINDOW_PIXMAP
-          if (have_name_window_pixmap (display))
-            {
-              x = cw->attrs.x;
-              y = cw->attrs.y;
-              wid = cw->attrs.width + cw->attrs.border_width * 2;
-              hei = cw->attrs.height + cw->attrs.border_width * 2;
-            }
-          else
-#endif
-            {
-              x = cw->attrs.x + cw->attrs.border_width;
-              y = cw->attrs.y + cw->attrs.border_width;
-              wid = cw->attrs.width;
-              hei = cw->attrs.height;
-            }
+          x = cw->attrs.x;
+          y = cw->attrs.y;
+          wid = cw->attrs.width + cw->attrs.border_width * 2;
+          hei = cw->attrs.height + cw->attrs.border_width * 2;
 
           XFixesSetPictureClipRegion (xdisplay, root_buffer,
                                       0, 0, paint_region);
@@ -1293,7 +1384,7 @@ paint_windows (MetaScreen   *screen,
 
               shadow_clip = XFixesCreateRegion (xdisplay, NULL, 0);
               XFixesSubtractRegion (xdisplay, shadow_clip, cw->border_clip,
-                                    cw->border_size);
+                                    cw->window_size);
               XFixesSetPictureClipRegion (xdisplay, root_buffer, 0, 0,
                                           shadow_clip);
 
@@ -1321,22 +1412,11 @@ paint_windows (MetaScreen   *screen,
           if (cw->mode == WINDOW_ARGB)
             {
               int x, y, wid, hei;
-#ifdef HAVE_NAME_WINDOW_PIXMAP
-              if (have_name_window_pixmap (display))
-                {
-                  x = cw->attrs.x;
-                  y = cw->attrs.y;
-                  wid = cw->attrs.width + cw->attrs.border_width * 2;
-                  hei = cw->attrs.height + cw->attrs.border_width * 2;
-                }
-              else
-#endif
-                {
-                  x = cw->attrs.x + cw->attrs.border_width;
-                  y = cw->attrs.y + cw->attrs.border_width;
-                  wid = cw->attrs.width;
-                  hei = cw->attrs.height;
-                }
+
+              x = cw->attrs.x;
+              y = cw->attrs.y;
+              wid = cw->attrs.width + cw->attrs.border_width * 2;
+              hei = cw->attrs.height + cw->attrs.border_width * 2;
 
               XRenderComposite (xdisplay, PictOpOver, cw->picture,
                                 cw->alpha_pict, root_buffer, 0, 0, 0, 0,
@@ -1548,23 +1628,18 @@ free_win (MetaCompWindow *cw,
   Display *xdisplay = meta_display_get_xdisplay (display);
   MetaCompScreen *info = meta_screen_get_compositor_data (cw->screen);
 
-#ifdef HAVE_NAME_WINDOW_PIXMAP
-  if (have_name_window_pixmap (display))
+  /* See comment in map_win */
+  if (cw->back_pixmap && destroy)
     {
-      /* See comment in map_win */
-      if (cw->back_pixmap && destroy)
-        {
-          XFreePixmap (xdisplay, cw->back_pixmap);
-          cw->back_pixmap = None;
-        }
+      XFreePixmap (xdisplay, cw->back_pixmap);
+      cw->back_pixmap = None;
+    }
 
-      if (cw->shaded_back_pixmap && destroy)
-        {
-          XFreePixmap (xdisplay, cw->shaded_back_pixmap);
-          cw->shaded_back_pixmap = None;
-        }
+  if (cw->shaded_back_pixmap && destroy)
+    {
+      XFreePixmap (xdisplay, cw->shaded_back_pixmap);
+      cw->shaded_back_pixmap = None;
     }
-#endif
 
   if (cw->picture)
     {
@@ -1596,6 +1671,12 @@ free_win (MetaCompWindow *cw,
       cw->border_size = None;
     }
 
+  if (cw->window_size)
+    {
+      XFixesDestroyRegion (xdisplay, cw->window_size);
+      cw->window_size = None;
+    }
+
   if (cw->border_clip)
     {
       XFixesDestroyRegion (xdisplay, cw->border_clip);
@@ -1638,7 +1719,6 @@ map_win (MetaDisplay *display,
   if (cw == NULL)
     return;
 
-#ifdef HAVE_NAME_WINDOW_PIXMAP
   /* The reason we deallocate this here and not in unmap
      is so that we will still have a valid pixmap for
      whenever the window is unmapped */
@@ -1653,7 +1733,6 @@ map_win (MetaDisplay *display,
       XFreePixmap (xdisplay, cw->shaded_back_pixmap);
       cw->shaded_back_pixmap = None;
     }
-#endif
 
   cw->attrs.map_state = IsViewable;
   cw->damaged = FALSE;
@@ -1847,11 +1926,8 @@ add_win (MetaScreen *screen,
 
   XSelectInput (xdisplay, xwindow, event_mask);
 
-
-#ifdef HAVE_NAME_WINDOW_PIXMAP
   cw->back_pixmap = None;
   cw->shaded_back_pixmap = None;
-#endif
 
   cw->damaged = FALSE;
   cw->shaped = is_shaped (display, xwindow);
@@ -1864,6 +1940,7 @@ add_win (MetaScreen *screen,
   cw->alpha_pict = None;
   cw->shadow_pict = None;
   cw->border_size = None;
+  cw->window_size = None;
   cw->extents = None;
   cw->shadow = None;
   cw->shadow_dx = 0;
@@ -2033,32 +2110,28 @@ resize_win (MetaCompWindow *cw,
 
   if (cw->attrs.width != width || cw->attrs.height != height)
     {
-#ifdef HAVE_NAME_WINDOW_PIXMAP
-      if (have_name_window_pixmap (display))
+      if (cw->shaded_back_pixmap)
         {
-          if (cw->shaded_back_pixmap)
-            {
-              XFreePixmap (xdisplay, cw->shaded_back_pixmap);
-              cw->shaded_back_pixmap = None;
-            }
+          XFreePixmap (xdisplay, cw->shaded_back_pixmap);
+          cw->shaded_back_pixmap = None;
+        }
 
-          if (cw->back_pixmap)
+      if (cw->back_pixmap)
+        {
+          /* If the window is shaded, we store the old backing pixmap
+             so we can return a proper image of the window */
+          if (cw->window && meta_window_is_shaded (cw->window))
             {
-              /* If the window is shaded, we store the old backing pixmap
-                 so we can return a proper image of the window */
-              if (cw->window && meta_window_is_shaded (cw->window))
-                {
-                  cw->shaded_back_pixmap = cw->back_pixmap;
-                  cw->back_pixmap = None;
-                }
-              else
-                {
-                  XFreePixmap (xdisplay, cw->back_pixmap);
-                  cw->back_pixmap = None;
-                }
+              cw->shaded_back_pixmap = cw->back_pixmap;
+              cw->back_pixmap = None;
+            }
+          else
+            {
+              XFreePixmap (xdisplay, cw->back_pixmap);
+              cw->back_pixmap = None;
             }
         }
-#endif
+
       if (cw->picture)
         {
           XRenderFreePicture (xdisplay, cw->picture);
@@ -2495,29 +2568,22 @@ show_overlay_window (MetaScreen *screen,
 {
   MetaDisplay *display = meta_screen_get_display (screen);
   Display *xdisplay = meta_display_get_xdisplay (display);
+  XserverRegion region;
 
-#ifdef HAVE_COW
-  if (have_cow (display))
-    {
-      XserverRegion region;
+  region = XFixesCreateRegion (xdisplay, NULL, 0);
 
-      region = XFixesCreateRegion (xdisplay, NULL, 0);
+  XFixesSetWindowShapeRegion (xdisplay, cow, ShapeBounding, 0, 0, 0);
+  XFixesSetWindowShapeRegion (xdisplay, cow, ShapeInput, 0, 0, region);
 
-      XFixesSetWindowShapeRegion (xdisplay, cow, ShapeBounding, 0, 0, 0);
-      XFixesSetWindowShapeRegion (xdisplay, cow, ShapeInput, 0, 0, region);
+  XFixesDestroyRegion (xdisplay, region);
 
-      XFixesDestroyRegion (xdisplay, region);
-
-      damage_screen (screen);
-    }
-#endif
+  damage_screen (screen);
 }
 
 static void
 hide_overlay_window (MetaScreen *screen,
                      Window      cow)
 {
-#ifdef HAVE_COW
   MetaDisplay *display = meta_screen_get_display (screen);
   Display *xdisplay = meta_display_get_xdisplay (display);
   XserverRegion region;
@@ -2525,7 +2591,6 @@ hide_overlay_window (MetaScreen *screen,
   region = XFixesCreateRegion (xdisplay, NULL, 0);
   XFixesSetWindowShapeRegion (xdisplay, cow, ShapeBounding, 0, 0, region);
   XFixesDestroyRegion (xdisplay, region);
-#endif
 }
 
 static Window
@@ -2537,17 +2602,8 @@ get_output_window (MetaScreen *screen)
 
   xroot = meta_screen_get_xroot (screen);
 
-#ifdef HAVE_COW
-  if (have_cow (display))
-    {
-      output = XCompositeGetOverlayWindow (xdisplay, xroot);
-      XSelectInput (xdisplay, output, ExposureMask);
-    }
-  else
-#endif
-    {
-      output = xroot;
-    }
+  output = XCompositeGetOverlayWindow (xdisplay, xroot);
+  XSelectInput (xdisplay, output, ExposureMask);
 
   return output;
 }
@@ -2684,9 +2740,7 @@ xrender_unmanage_screen (MetaCompositor *compositor,
                                   CompositeRedirectManual);
   meta_screen_unset_cm_selection (screen);
 
-#ifdef HAVE_COW
   XCompositeReleaseOverlayWindow (xdisplay, info->output);
-#endif
 
   g_free (info);
 
@@ -2817,10 +2871,8 @@ xrender_process_event (MetaCompositor *compositor,
     default:
       if (event->type == meta_display_get_damage_event_base (xrc->display) + XDamageNotify)
         process_damage (xrc, (XDamageNotifyEvent *) event);
-#ifdef HAVE_SHAPE
       else if (event->type == meta_display_get_shape_event_base (xrc->display) + ShapeNotify)
         process_shape (xrc, (XShapeEvent *) event);
-#endif /* HAVE_SHAPE */
       else
         {
           meta_error_trap_pop (xrc->display, FALSE);
@@ -2852,17 +2904,10 @@ xrender_get_window_pixmap (MetaCompositor *compositor,
   if (cw == NULL)
     return None;
 
-#ifdef HAVE_NAME_WINDOW_PIXMAP
-  if (have_name_window_pixmap (meta_window_get_display (window)))
-    {
-      if (meta_window_is_shaded (window))
-        return cw->shaded_back_pixmap;
-      else
-        return cw->back_pixmap;
-    }
+  if (meta_window_is_shaded (window))
+    return cw->shaded_back_pixmap;
   else
-#endif
-    return None;
+    return cw->back_pixmap;
 #endif
 }
 
