@@ -269,8 +269,7 @@ meta_core_user_lower_and_unfocus (Display *xdisplay,
 
   meta_window_lower (window);
 
-  if (meta_prefs_get_focus_mode () == G_DESKTOP_FOCUS_MODE_CLICK &&
-      meta_prefs_get_raise_on_click ())
+  if (meta_prefs_get_raise_on_click ())
     {
       /* Move window to the back of the focusing workspace's MRU list.
        * Do extra sanity checks to avoid possible race conditions.
@@ -813,3 +812,9 @@ meta_invalidate_default_icons (void)
   g_slist_free (windows);
 }
 
+void
+meta_retheme_all (void)
+{
+  if (meta_get_display ())
+    meta_display_retheme_all ();
+}
