@@ -262,6 +262,16 @@ cardinal_list_from_results (GetPropertyResults *results,
   *n_cardinals_p = results->n_items;
   results->prop = NULL;
 
+#if GLIB_SIZEOF_LONG == 8
+  /* Xlib sign-extends format=32 items, but we want them unsigned */
+  {
+    int i;
+
+    for (i = 0; i < *n_cardinals_p; i++)
+      (*cardinals_p)[i] = (*cardinals_p)[i] & 0xffffffff;
+  }
+#endif
+
   return TRUE;
 }
 
@@ -552,7 +562,6 @@ window_from_results (GetPropertyResults *results,
   return TRUE;
 }
 
-#ifdef HAVE_XSYNC
 static gboolean
 counter_from_results (GetPropertyResults *results,
                       XSyncCounter       *counter_p)
@@ -568,7 +577,6 @@ counter_from_results (GetPropertyResults *results,
 
   return TRUE;
 }
-#endif
 
 gboolean
 meta_prop_get_window (MetaDisplay *display,
@@ -606,6 +614,10 @@ cardinal_with_atom_type_from_results (GetPropertyResults *results,
     return FALSE;
 
   *cardinal_p = *(gulong*) results->prop;
+#if GLIB_SIZEOF_LONG == 8
+  /* Xlib sign-extends format=32 items, but we want them unsigned */
+  *cardinal_p &= 0xffffffff;
+#endif
   XFree (results->prop);
   results->prop = NULL;
 
@@ -630,6 +642,29 @@ meta_prop_get_cardinal_with_atom_type (MetaDisplay   *display,
   return cardinal_with_atom_type_from_results (&results, prop_type, cardinal_p);
 }
 
+static char *
+text_property_to_utf8 (Display             *xdisplay,
+                       const XTextProperty *prop)
+{
+  char *ret = NULL;
+  char **local_list = NULL;
+  int count = 0;
+  int res;
+
+  res = XmbTextPropertyToTextList (xdisplay, prop, &local_list, &count);
+  if (res == XNoMemory || res == XLocaleNotSupported || res == XConverterNotFound)
+    goto out;
+
+  if (count == 0)
+    goto out;
+
+  ret = g_strdup (local_list[0]);
+
+  out:
+    meta_XFree (local_list);
+    return ret;
+}
+
 static gboolean
 text_property_from_results (GetPropertyResults *results,
                             char              **utf8_str_p)
@@ -644,8 +679,7 @@ text_property_from_results (GetPropertyResults *results,
   tp.format = results->format;
   tp.nitems = results->n_items;
 
-  *utf8_str_p = meta_text_property_to_utf8 (results->display->xdisplay,
-                                            &tp);
+  *utf8_str_p = text_property_to_utf8 (results->display->xdisplay, &tp);
 
   if (tp.value != NULL)
     XFree (tp.value);
@@ -1150,18 +1184,9 @@ meta_prop_get_values (MetaDisplay   *display,
             values[i].type = META_PROP_VALUE_INVALID;
           break;
         case META_PROP_VALUE_SYNC_COUNTER:
-#ifdef HAVE_XSYNC
           if (!counter_from_results (&results,
                                      &values[i].v.xcounter))
             values[i].type = META_PROP_VALUE_INVALID;
-#else
-          values[i].type = META_PROP_VALUE_INVALID;
-          if (results.prop)
-            {
-              XFree (results.prop);
-              results.prop = NULL;
-            }
-#endif
           break;
         }
 
