@@ -66,9 +66,9 @@ void meta_compositor_end_move (MetaCompositor *compositor,
 void meta_compositor_free_window (MetaCompositor *compositor,
                                   MetaWindow *window);
 
+void meta_compositor_maximize_window   (MetaCompositor *compositor,
+                                        MetaWindow     *window);
+void meta_compositor_unmaximize_window (MetaCompositor *compositor,
+                                        MetaWindow     *window);
+
 #endif
-
-
-
-
-
