@@ -22,8 +22,12 @@
 
 #include <X11/Xlib.h>
 
+#include "common.h"
 #include "types.h"
 
 Window meta_frame_get_xwindow (MetaFrame *frame);
 
+void   meta_frame_calc_borders (MetaFrame        *frame,
+                                MetaFrameBorders *borders);
+
 #endif
