@@ -837,7 +837,7 @@ main (int argc, char **argv)
 {
   GtkWidget *window;
   GtkWidget *collection;
-  GtkStyleContext *style;
+  MetaStyleInfo *style_info;
   PangoFontDescription *font_desc;
   GError *err;
   clock_t start, end;
@@ -913,10 +913,12 @@ main (int argc, char **argv)
                     G_CALLBACK (gtk_main_quit), NULL);
 
   gtk_widget_realize (window);
-  style = gtk_widget_get_style_context (window);
-  gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL, "font", &font_desc, NULL);
 
-  g_assert (style);
+  style_info = meta_theme_create_style_info (gtk_widget_get_screen (window), NULL);
+  gtk_style_context_get (style_info->styles[META_STYLE_ELEMENT_FRAME],
+                         GTK_STATE_FLAG_NORMAL, "font", &font_desc, NULL);
+  meta_style_info_unref (style_info);
+
   g_assert (font_desc);
 
   notebook = gtk_notebook_new ();
@@ -986,14 +988,14 @@ get_flags (GtkWidget *widget)
 }
 
 static int
-get_text_height (GtkWidget *widget)
+get_text_height (GtkWidget     *widget,
+                 MetaStyleInfo *style_info)
 {
-  GtkStyleContext      *style;
   PangoFontDescription *font_desc;
   int                   text_height;
 
-  style = gtk_widget_get_style_context (widget);
-  gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL, "font", &font_desc, NULL);
+  gtk_style_context_get (style_info->styles[META_STYLE_ELEMENT_FRAME],
+                         GTK_STATE_FLAG_NORMAL, "font", &font_desc, NULL);
   text_height = meta_pango_font_desc_get_text_height (font_desc, gtk_widget_get_pango_context (widget));
   pango_font_description_free (font_desc);
 
@@ -1014,6 +1016,7 @@ static void
 run_theme_benchmark (void)
 {
   GtkWidget* widget;
+  MetaStyleInfo *style_info;
   cairo_surface_t *pixmap;
   MetaFrameBorders borders;
   MetaButtonState button_states[META_BUTTON_TYPE_LAST] =
@@ -1038,9 +1041,12 @@ run_theme_benchmark (void)
   widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_widget_realize (widget);
 
+  style_info = meta_theme_create_style_info (gtk_widget_get_screen (widget), NULL);
+
   meta_theme_get_frame_borders (global_theme,
+                                style_info,
                                 META_FRAME_TYPE_NORMAL,
-                                get_text_height (widget),
+                                get_text_height (widget, style_info),
                                 get_flags (widget),
                                 &borders);
 
@@ -1077,18 +1083,18 @@ run_theme_benchmark (void)
        */
       pixmap = gdk_window_create_similar_surface (gtk_widget_get_window (widget),
                                                   CAIRO_CONTENT_COLOR,
-                                                  client_width + borders.visible.left + borders.visible.right,
-                                                  client_height + borders.visible.top + borders.visible.bottom);
+                                                  client_width + borders.total.left + borders.total.right,
+                                                  client_height + borders.total.top + borders.total.bottom);
       cr = cairo_create (pixmap);
 
       meta_theme_draw_frame (global_theme,
-                             widget,
+                             style_info,
                              cr,
                              META_FRAME_TYPE_NORMAL,
                              get_flags (widget),
                              client_width, client_height,
                              layout,
-                             get_text_height (widget),
+                             get_text_height (widget, style_info),
                              &button_layout,
                              button_states,
                              meta_preview_get_mini_icon (),
@@ -1116,6 +1122,7 @@ run_theme_benchmark (void)
 
   g_timer_destroy (timer);
   g_object_unref (G_OBJECT (layout));
+  meta_style_info_unref (style_info);
   gtk_widget_destroy (widget);
 
 #undef ITERATIONS
